# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class BillRunSchedule(BaseModel):
    """
    Container for information about the scheduled bill run. 
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    repeat_from: Optional[date] = Field(default=None, description="The start date of the scheduled bill run. ", alias="repeatFrom")
    repeat_to: Optional[date] = Field(default=None, description="The end date of of the scheduled bill run. ", alias="repeatTo")
    repeat_type: Optional[StrictStr] = Field(default=None, description="The repeat type of the bill run. ", alias="repeatType")
    run_time: Optional[StrictInt] = Field(default=None, description="The scheduled run time (hour) of day.  **Values:** 0 - 23 ", alias="runTime")
    weekly_on_day: Optional[List[StrictStr]] = Field(default=None, description="The repeat day in a week. ", alias="weeklyOnDay")
    monthly_on_end_of_month: Optional[StrictBool] = Field(default=None, description="Whether to schedule monthly bill run on the end of month or the specific day of month. This field is available only when repeatType is set to monthly and repeatFrom is set to the end of month.  For example: - When repeatFrom = '2024-04-30' and monthlyOnEndOfMonth = true, next bill run will be scheduled on 2024-05-31 - When repeatFrom = '2024-04-30' and monthlyOnEndOfMonth = false, next bill run will be scheduled on 2024-05-30 ", alias="monthlyOnEndOfMonth")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["repeatFrom", "repeatTo", "repeatType", "runTime", "weeklyOnDay", "monthlyOnEndOfMonth"]

    @field_validator('repeat_type')
    def repeat_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['None', 'Daily', 'Weekly', 'Monthly']):
            raise ValueError("must be one of enum values ('None', 'Daily', 'Weekly', 'Monthly')")
        return value

    @field_validator('weekly_on_day')
    def weekly_on_day_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in set(['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']):
                raise ValueError("each list item must be one of ('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BillRunSchedule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BillRunSchedule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "repeatFrom": obj.get("repeatFrom"),
            "repeatTo": obj.get("repeatTo"),
            "repeatType": obj.get("repeatType"),
            "runTime": obj.get("runTime"),
            "weeklyOnDay": obj.get("weeklyOnDay"),
            "monthlyOnEndOfMonth": obj.get("monthlyOnEndOfMonth")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
