# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class ContactSnapshot(BaseModel):
    """
    ContactSnapshot
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    address1: Optional[StrictStr] = Field(default=None, description="The first line for the address of the contact, which is often a street address or business name. ")
    address2: Optional[StrictStr] = Field(default=None, description="The second line for the address of the contact, which is mostly the locality. ")
    city: Optional[StrictStr] = Field(default=None, description="The city for the address of the contact. ")
    contact_id: Optional[StrictStr] = Field(default=None, description="The Zuora ID of the contact who the snapshot belongs to. ", alias="contactId")
    country: Optional[StrictStr] = Field(default=None, description="The country for the address of the contact. ")
    county: Optional[StrictStr] = Field(default=None, description="The county for the address of the contact. The field value might optionally be used by Zuora Tax to calculate county tax. ")
    description: Optional[StrictStr] = Field(default=None, description="A description of the contact. ")
    fax: Optional[StrictStr] = Field(default=None, description="The fax number of the contact. ")
    first_name: Optional[StrictStr] = Field(default=None, description="The first name of the contact. ", alias="firstName")
    home_phone: Optional[StrictStr] = Field(default=None, description="The home phone number of the contact. ", alias="homePhone")
    id: Optional[StrictStr] = Field(default=None, description="The unique ID of the contact snapshot. ")
    last_name: Optional[StrictStr] = Field(default=None, description="The last name of the contact. ", alias="lastName")
    mobile_phone: Optional[StrictStr] = Field(default=None, description="The mobile phone number of the contact. ", alias="mobilePhone")
    nickname: Optional[StrictStr] = Field(default=None, description="A nickname for the contact. ")
    other_phone: Optional[StrictStr] = Field(default=None, description="An additional phone number for the contact. ", alias="otherPhone")
    other_phone_type: Optional[StrictStr] = Field(default=None, alias="otherPhoneType")
    personal_email: Optional[StrictStr] = Field(default=None, description="The personal email address of the contact. ", alias="personalEmail")
    postal_code: Optional[StrictStr] = Field(default=None, description="The postal code for the address of the contact. ", alias="postalCode")
    state: Optional[StrictStr] = Field(default=None, description="The state or province for the address of the contact. ")
    tax_region: Optional[StrictStr] = Field(default=None, description="If using Zuora Tax rules. ", alias="taxRegion")
    work_email: Optional[StrictStr] = Field(default=None, description="The business email address of the contact. ", alias="workEmail")
    work_phone: Optional[StrictStr] = Field(default=None, description="The business email address of the contact. ", alias="workPhone")
    as_bill_to: Optional[StrictBool] = Field(default=None, description="Indicates the contact can as a bill to. Need Permission 'ShipToContactSupport' ", alias="asBillTo")
    as_sold_to: Optional[StrictBool] = Field(default=None, description="Indicates the contact can as a sold to. Need Permission 'ShipToContactSupport' ", alias="asSoldTo")
    as_ship_to: Optional[StrictBool] = Field(default=None, description="Indicates the contact can as a ship to. Need Permission 'ShipToContactSupport' ", alias="asShipTo")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["address1", "address2", "city", "contactId", "country", "county", "description", "fax", "firstName", "homePhone", "id", "lastName", "mobilePhone", "nickname", "otherPhone", "otherPhoneType", "personalEmail", "postalCode", "state", "taxRegion", "workEmail", "workPhone", "asBillTo", "asSoldTo", "asShipTo"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ContactSnapshot from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ContactSnapshot from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "address1": obj.get("address1"),
            "address2": obj.get("address2"),
            "city": obj.get("city"),
            "contactId": obj.get("contactId"),
            "country": obj.get("country"),
            "county": obj.get("county"),
            "description": obj.get("description"),
            "fax": obj.get("fax"),
            "firstName": obj.get("firstName"),
            "homePhone": obj.get("homePhone"),
            "id": obj.get("id"),
            "lastName": obj.get("lastName"),
            "mobilePhone": obj.get("mobilePhone"),
            "nickname": obj.get("nickname"),
            "otherPhone": obj.get("otherPhone"),
            "otherPhoneType": obj.get("otherPhoneType"),
            "personalEmail": obj.get("personalEmail"),
            "postalCode": obj.get("postalCode"),
            "state": obj.get("state"),
            "taxRegion": obj.get("taxRegion"),
            "workEmail": obj.get("workEmail"),
            "workPhone": obj.get("workPhone"),
            "asBillTo": obj.get("asBillTo"),
            "asSoldTo": obj.get("asSoldTo"),
            "asShipTo": obj.get("asShipTo")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
