# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from zuora_sdk.models.create_payment_method_bank_transfer_account_holder_info import CreatePaymentMethodBankTransferAccountHolderInfo
from zuora_sdk.models.create_payment_method_cardholder_info import CreatePaymentMethodCardholderInfo
from zuora_sdk.models.payment_method_ach_bank_account_type import PaymentMethodACHBankAccountType
from zuora_sdk.models.stored_credential_profile_action import StoredCredentialProfileAction
from zuora_sdk.models.stored_credential_profile_consent_agreement_src import StoredCredentialProfileConsentAgreementSrc
from zuora_sdk.models.stored_credential_profile_type import StoredCredentialProfileType
from typing import Optional, Set
from typing_extensions import Self

class CreateAccountPaymentMethod(BaseModel):
    """
    Container for information on payment methods to associate with this account. To associate a credit card payment method when creating an account, Zuora recommends that you use this field instead of the `creditCard` field.  Note that you can not use this field together with `creditCard` or `hpmCreditCardPaymentMethodId` to associate a payment method to an account. 
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    account_key: Optional[StrictStr] = Field(default=None, description="Internal ID of the customer account that will own the payment method. ", alias="accountKey")
    auth_gateway: Optional[StrictStr] = Field(default=None, description="Internal ID of the payment gateway that Zuora will use to authorize the payments that are made with the payment method.  If you do not set this field, Zuora will use one of the following payment gateways instead:  * The default payment gateway of the customer account that owns the payment method, if the `accountKey` field is set. * The default payment gateway of your Zuora tenant, if the `accountKey` field is not set. ", alias="authGateway")
    ip_address: Optional[StrictStr] = Field(default=None, description="The IPv4 or IPv6 information of the user when the payment method is created or updated. Some gateways use this field for fraud prevention. If this field is passed to Zuora, Zuora directly passes it to gateways.   If the IP address length is beyond 45 characters, a validation error occurs.  For validating SEPA payment methods on Stripe v2, this field is required. ", alias="ipAddress")
    make_default: Optional[StrictBool] = Field(default=False, description="Specifies whether the payment method will be the default payment method of the customer account that owns the payment method. Only applicable if the `accountKey` field is set. ", alias="makeDefault")
    address_line1: Optional[StrictStr] = Field(default=None, description="First address line, 255 characters or less. ", alias="addressLine1")
    address_line2: Optional[StrictStr] = Field(default=None, description="Second address line, 255 characters or less. ", alias="addressLine2")
    bank_aba_code: Optional[StrictStr] = Field(default=None, description="The nine-digit routing number or ABA number used by banks. This field is only required if the `type` field is set to `ACH`. ", alias="bankABACode")
    bank_account_name: Optional[Annotated[str, Field(strict=True, max_length=70)]] = Field(default=None, description="The name of the account holder, which can be either a person or a company. This field is only required if the `type` field is set to `ACH`.  For ACH payment methods on the BlueSnap integration, see [Overview of BlueSnap gateway integration](https://knowledgecenter.zuora.com/Zuora_Billing/Billing_and_Payments/M_Payment_Gateways/Supported_Payment_Gateways/BlueSnap_Gateway/Overview_of_BlueSnap_gateway_integration#Payer_Name_Extraction) for more information about how Zuora splits the string in this field into two parts and passes them to BlueSnap's `firstName` and `lastName` fields. ", alias="bankAccountName")
    bank_account_number: Optional[Annotated[str, Field(strict=True, max_length=70)]] = Field(default=None, description="The bank account number associated with the ACH payment. This field is only required if the `type` field is set to `ACH`. However, for creating tokenized ACH payment methods on  Stripe v2, this field is optional if the `tokens` and `bankAccountMaskNumber` fields are specified. ", alias="bankAccountNumber")
    bank_account_mask_number: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="The masked bank account number associated with the ACH payment. This field is only required if the ACH payment method is created using tokens. ", alias="bankAccountMaskNumber")
    bank_account_type: Optional[StrictStr] = Field(default=None, alias="bankAccountType")
    bank_name: Optional[Annotated[str, Field(strict=True, max_length=70)]] = Field(default=None, description="The name of the bank where the ACH payment account is held. This field is only required if the `type` field is set to `ACH`.  When creating an ACH payment method on Adyen, this field is required by Zuora but it is not required by Adyen. To create the ACH payment method successfully, specify a real value for this field if you can. If it is not possible to get the real value for it, specify a dummy value. ", alias="bankName")
    city: Optional[StrictStr] = Field(default=None, description="City, 40 characters or less.  It is recommended to provide the city and country information when creating a payment method. The information will be used to process payments. If the information is not provided during payment method creation, the city and country data will be missing during payment processing.      ")
    country: Optional[StrictStr] = Field(default=None, description="Country, must be a valid country name or abbreviation.  See [Country Names and Their ISO Standard 2- and 3-Digit Codes](https://knowledgecenter.zuora.com/BB_Introducing_Z_Business/D_Country%2C_State%2C_and_Province_Codes/A_Country_Names_and_Their_ISO_Codes) for the list of supported country names and abbreviations.  It is recommended to provide the city and country information when creating a payment method. The information will be used to process payments. If the information is not provided during payment method creation, the city and country data will be missing during payment processing. ")
    phone: Optional[StrictStr] = Field(default=None, description="Phone number, 40 characters or less. ")
    state: Optional[StrictStr] = Field(default=None, description="State, must be a valid state name or 2-character abbreviation.  See [United States Standard State Codes](https://knowledgecenter.zuora.com/BB_Introducing_Z_Business/D_Country%2C_State%2C_and_Province_Codes/B_State_Names_and_2-Digit_Codes) and [Canadian Standard Province Codes](https://knowledgecenter.zuora.com/BB_Introducing_Z_Business/D_Country%2C_State%2C_and_Province_Codes/C_Canadian_Province_Names_and_2-Digit_Codes) for the list of supported names and abbreviations. ")
    zip_code: Optional[StrictStr] = Field(default=None, description="Zip code, 20 characters or less. ", alias="zipCode")
    iban: Optional[StrictStr] = Field(default=None, description="The International Bank Account Number. This field is required if the `type` field is set to `SEPA`. ", alias="IBAN")
    account_holder_info: Optional[CreatePaymentMethodBankTransferAccountHolderInfo] = Field(default=None, alias="accountHolderInfo")
    account_number: Optional[StrictStr] = Field(default=None, description="The number of the customer's bank account. This field is required for the following bank transfer payment methods:   - Direct Entry AU (`Becs`)   - Direct Debit NZ (`Becsnz`)   - Direct Debit UK (`Bacs`)   - Denmark Direct Debit (`Betalingsservice`)   - Sweden Direct Debit (`Autogiro`)   - Canadian Pre-Authorized Debit (`PAD`) ", alias="accountNumber")
    bank_code: Optional[StrictStr] = Field(default=None, description="The sort code or number that identifies the bank. This is also known as the sort code. This field is required for the following bank transfer payment methods:   - Direct Debit UK (`Bacs`)   - Denmark Direct Debit (`Betalingsservice`)   - Direct Debit NZ (`Becsnz`)   - Canadian Pre-Authorized Debit (`PAD`) ", alias="bankCode")
    branch_code: Optional[StrictStr] = Field(default=None, description="The branch code of the bank used for direct debit. This field is required for the following bank transfer payment methods:   - Sweden Direct Debit (`Autogiro`)   - Direct Entry AU (`Becs`)   - Direct Debit NZ (`Becsnz`)   - Canadian Pre-Authorized Debit (`PAD`) ", alias="branchCode")
    business_identification_code: Optional[StrictStr] = Field(default=None, description="The BIC code used for SEPA. ", alias="businessIdentificationCode")
    currency_code: Optional[StrictStr] = Field(default=None, description="The currency used for payment method authorization.  If this field is not specified, `currency` specified for the account is used for payment method authorization. If no currency is specified for the account, the default currency of the account is then used. ", alias="currencyCode")
    identity_number: Optional[StrictStr] = Field(default=None, description="The identity number of the customer. This field is required for the following bank transfer payment methods:   - Denmark Direct Debit (`Betalingsservice`)   - Sweden Direct Debit (`Autogiro`) ", alias="identityNumber")
    card_holder_info: Optional[CreatePaymentMethodCardholderInfo] = Field(default=None, alias="cardHolderInfo")
    card_number: Optional[StrictStr] = Field(default=None, description="Credit card number. This field is required if `type` is set to `CreditCard`. However, for creating tokenized credit card payment methods,  this field is optional if the `tokens` and `cardMaskNumber` fields are specified. ", alias="cardNumber")
    card_mask_number: Optional[StrictStr] = Field(default=None, description="The masked card number associated with the credit card payment. This field is only required if the credit card payment method is created using tokens.  ", alias="cardMaskNumber")
    card_type: Optional[StrictStr] = Field(default=None, description="The type of the credit card. This field is required if `type` is set to `CreditCard`.  Possible values include `Visa`, `MasterCard`, `AmericanExpress`, `Discover`, `JCB`, and `Diners`. For more information about credit card types supported by different payment gateways, see [Supported Payment Gateways](https://knowledgecenter.zuora.com/CB_Billing/M_Payment_Gateways/Supported_Payment_Gateways). ", alias="cardType")
    check_duplicated: Optional[StrictBool] = Field(default=None, description="Indicates whether the duplication check is performed when you create a new credit card payment method. The default value is `false`.  With this field set to `true`, Zuora will check all active payment methods associated with the same billing account to ensure that no duplicate credit card payment methods are created. An error is returned if a duplicate payment method is found.          The following fields are used for the duplication check:   - `cardHolderName`   - `expirationMonth`   - `expirationYear`   - `creditCardMaskNumber`. It is the masked credit card number generated by Zuora. For example:     ```     ************1234     ``` ", alias="checkDuplicated")
    expiration_month: Optional[StrictInt] = Field(default=None, description="One or two digit expiration month (1-12) of the credit card. This field is required if `type` is set to `CreditCard`. However, for creating tokenized credit card payment methods,  this field is optional if the `tokens` and `cardMaskNumber` fields are specified. ", alias="expirationMonth")
    expiration_year: Optional[StrictInt] = Field(default=None, description="Four-digit expiration year of the credit card. This field is required if `type` is set to `CreditCard`. However, for creating tokenized credit card payment methods,  this field is optional if the `tokens` and `cardMaskNumber` fields are specified. ", alias="expirationYear")
    mit_consent_agreement_ref: Optional[Annotated[str, Field(strict=True, max_length=128)]] = Field(default=None, description="Specifies your reference for the stored credential consent agreement that you have established with the customer. Only applicable if you set the `mitProfileAction` field. ", alias="mitConsentAgreementRef")
    mit_consent_agreement_src: Optional[StrictStr] = Field(default=None, alias="mitConsentAgreementSrc")
    mit_network_transaction_id: Optional[Annotated[str, Field(strict=True, max_length=128)]] = Field(default=None, description="Specifies the ID of a network transaction. Only applicable if you set the `mitProfileAction` field to `Persist`. ", alias="mitNetworkTransactionId")
    mit_profile_action: Optional[StrictStr] = Field(default=None, alias="mitProfileAction")
    mit_profile_agreed_on: Optional[date] = Field(default=None, description="The date on which the profile is agreed. The date format is `yyyy-mm-dd`. ", alias="mitProfileAgreedOn")
    mit_profile_type: Optional[StrictStr] = Field(default=None, alias="mitProfileType")
    security_code: Optional[StrictStr] = Field(default=None, description="CVV or CVV2 security code of the credit card.  To ensure PCI compliance, this value is not stored and cannot be queried. ", alias="securityCode")
    credit_card_mask_number: Optional[Annotated[str, Field(strict=True, max_length=19)]] = Field(default=None, description="The masked credit card number, such as: ``` *********1112 ```  This field is specific for the CC Reference Transaction payment method. It is an optional field that you can use to distinguish different CC Reference Transaction payment methods.  Though there are no special restrictions on the input string, it is highly recommended to specify a card number that is masked. ", alias="creditCardMaskNumber")
    second_token_id: Optional[StrictStr] = Field(default=None, description="A gateway unique identifier that replaces sensitive payment method data.   `secondTokenId` is conditionally required only when `tokenId` is being used to represent a gateway customer profile. `secondTokenId` is used in the CC Reference Transaction payment method. ", alias="secondTokenId")
    token_id: Optional[StrictStr] = Field(default=None, description="A gateway unique identifier that replaces sensitive payment method data or represents a gateway's unique customer profile. `tokenId` is required for the CC Reference Transaction payment method.  When `tokenId` is used to represent a customer profile, `secondTokenId` is conditionally required for representing the underlying tokenized payment method.  The values for the `tokenId` and `secondTokenId` fields differ for gateways. For more information, see the Knowledge Center article specific to each gateway that supports the CC Reference Transaction payment method.  **Note:** When creating an ACH payment method, if you need to pass in tokenized information, use the `mandateId` instead of `tokenId` field. ", alias="tokenId")
    baid: Optional[StrictStr] = Field(default=None, description="ID of a PayPal billing agreement. For example, I-1TJ3GAGG82Y9. ", alias="BAID")
    email: Optional[StrictStr] = Field(default=None, description="Email address associated with the payment method. This field is supported for the following payment methods:   - PayPal payment methods. This field is required for creating any of the following PayPal payment methods.     - PayPal Express Checkout     - PayPal Adaptive     - PayPal Commerce Platform   - Apple Pay and Google Pay payment methods on Adyen v2.0. This field will be passed to Adyen as `shopperEmail`. ")
    preapproval_key: Optional[StrictStr] = Field(default=None, description="The PayPal preapproval key. ", alias="preapprovalKey")
    apple_payment_data: Optional[StrictStr] = Field(default=None, description="This field is specific for setting up Apple Pay for Adyen to include payload with Apple Pay token or Apple payment data. This information should be stringified. For more information, see [Set up Adyen Apple Pay](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/L_Payment_Methods/Payment_Method_Types/Apple_Pay_on_Web/Set_up_Adyen_Apple_Pay). ", alias="applePaymentData")
    google_payment_token: Optional[StrictStr] = Field(default=None, description="This field is specific for setting up Google Pay for Adyen and Chase gateway integrations to specify the stringified Google Pay token. For more information, see [Set up Adyen Google Pay](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/L_Payment_Methods/Payment_Method_Types/Set_up_Adyen_Google_Pay) and [Set up Google Pay on Chase](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/L_Payment_Methods/Payment_Method_Types/Set_up_Google_Pay_on_Chase). ", alias="googlePaymentToken")
    type: StrictStr = Field(description="Type of payment method. Possible values include:  * `CreditCard` - Credit card payment method. * `CreditCardReferenceTransaction` - Credit Card Reference Transaction. See [Supported payment methods](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/L_Payment_Methods/Supported_Payment_Methods) for payment gateways that support this type of payment method. * `ACH` - ACH payment method. * `SEPA` - Single Euro Payments Area. * `Betalingsservice` - Direct Debit DK. * `Autogiro` - Direct Debit SE. * `Bacs` - Direct Debit UK. * `Becs` - Direct Entry AU. * `Becsnz` - Direct Debit NZ. * `PAD` - Pre-Authorized Debit. * `PayPalCP` - PayPal Commerce Platform payment method. Use this type if you are using a [PayPal Commerce Platform Gateway](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/M_Payment_Gateways/Supported_Payment_Gateways/PayPal_Commerce_Platform_Gateway) instance. * `PayPalEC` - PayPal Express Checkout payment method. Use this type if you are using a [PayPal Payflow Pro Gateway](https://knowledgecenter.zuora.com/CB_Billing/M_Payment_Gateways/Supported_Payment_Gateways/PayPal_Payflow_Pro%2C_Website_Payments_Payflow_Edition%2C_Website_Pro_Payment_Gateway) instance. * `PayPalNativeEC` - PayPal Native Express Checkout payment method. Use this type if you are using a [PayPal Express Checkout Gateway](https://knowledgecenter.zuora.com/CB_Billing/M_Payment_Gateways/Supported_Payment_Gateways/PayPal_Express_Checkout_Gateway) instance. * `PayPalAdaptive` - PayPal Adaptive payment method. Use this type if you are using a [PayPal Adaptive Payment Gateway](https://knowledgecenter.zuora.com/CB_Billing/M_Payment_Gateways/Supported_Payment_Gateways/PayPal_Adaptive_Payments_Gateway) instance. * `AdyenApplePay` - Apple Pay on Adyen Integration v2.0. See [Set up Adyen Apple Pay](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/L_Payment_Methods/Payment_Method_Types/Apple_Pay_on_Web/Set_up_Adyen_Apple_Pay) for details. * `AdyenGooglePay` - Google Pay on Adyen Integration v2.0. See [Set up Adyen Google Pay](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/L_Payment_Methods/Payment_Method_Types/Set_up_Adyen_Google_Pay) for details. * `GooglePay` - Google Pay on Chase Paymentech Orbital gateway integration. See [Set up Google Pay on Chase](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/L_Payment_Methods/Payment_Method_Types/Set_up_Google_Pay_on_Chase) for details. * You can also specify a custom payment method type. See [Set up custom payment gateways and payment methods](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/MB_Set_up_custom_payment_gateways_and_payment_methods) for details. ")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["accountKey", "authGateway", "ipAddress", "makeDefault", "addressLine1", "addressLine2", "bankABACode", "bankAccountName", "bankAccountNumber", "bankAccountMaskNumber", "bankAccountType", "bankName", "city", "country", "phone", "state", "zipCode", "IBAN", "accountHolderInfo", "accountNumber", "bankCode", "branchCode", "businessIdentificationCode", "currencyCode", "identityNumber", "cardHolderInfo", "cardNumber", "cardMaskNumber", "cardType", "checkDuplicated", "expirationMonth", "expirationYear", "mitConsentAgreementRef", "mitConsentAgreementSrc", "mitNetworkTransactionId", "mitProfileAction", "mitProfileAgreedOn", "mitProfileType", "securityCode", "creditCardMaskNumber", "secondTokenId", "tokenId", "BAID", "email", "preapprovalKey", "applePaymentData", "googlePaymentToken", "type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateAccountPaymentMethod from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of account_holder_info
        if self.account_holder_info:
            _dict['accountHolderInfo'] = self.account_holder_info.to_dict()
        # override the default output from pydantic by calling `to_dict()` of card_holder_info
        if self.card_holder_info:
            _dict['cardHolderInfo'] = self.card_holder_info.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateAccountPaymentMethod from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "accountKey": obj.get("accountKey"),
            "authGateway": obj.get("authGateway"),
            "ipAddress": obj.get("ipAddress"),
            "makeDefault": obj.get("makeDefault") if obj.get("makeDefault") is not None else False,
            "addressLine1": obj.get("addressLine1"),
            "addressLine2": obj.get("addressLine2"),
            "bankABACode": obj.get("bankABACode"),
            "bankAccountName": obj.get("bankAccountName"),
            "bankAccountNumber": obj.get("bankAccountNumber"),
            "bankAccountMaskNumber": obj.get("bankAccountMaskNumber"),
            "bankAccountType": obj.get("bankAccountType"),
            "bankName": obj.get("bankName"),
            "city": obj.get("city"),
            "country": obj.get("country"),
            "phone": obj.get("phone"),
            "state": obj.get("state"),
            "zipCode": obj.get("zipCode"),
            "IBAN": obj.get("IBAN"),
            "accountHolderInfo": CreatePaymentMethodBankTransferAccountHolderInfo.from_dict(obj["accountHolderInfo"]) if obj.get("accountHolderInfo") is not None else None,
            "accountNumber": obj.get("accountNumber"),
            "bankCode": obj.get("bankCode"),
            "branchCode": obj.get("branchCode"),
            "businessIdentificationCode": obj.get("businessIdentificationCode"),
            "currencyCode": obj.get("currencyCode"),
            "identityNumber": obj.get("identityNumber"),
            "cardHolderInfo": CreatePaymentMethodCardholderInfo.from_dict(obj["cardHolderInfo"]) if obj.get("cardHolderInfo") is not None else None,
            "cardNumber": obj.get("cardNumber"),
            "cardMaskNumber": obj.get("cardMaskNumber"),
            "cardType": obj.get("cardType"),
            "checkDuplicated": obj.get("checkDuplicated"),
            "expirationMonth": obj.get("expirationMonth"),
            "expirationYear": obj.get("expirationYear"),
            "mitConsentAgreementRef": obj.get("mitConsentAgreementRef"),
            "mitConsentAgreementSrc": obj.get("mitConsentAgreementSrc"),
            "mitNetworkTransactionId": obj.get("mitNetworkTransactionId"),
            "mitProfileAction": obj.get("mitProfileAction"),
            "mitProfileAgreedOn": obj.get("mitProfileAgreedOn"),
            "mitProfileType": obj.get("mitProfileType"),
            "securityCode": obj.get("securityCode"),
            "creditCardMaskNumber": obj.get("creditCardMaskNumber"),
            "secondTokenId": obj.get("secondTokenId"),
            "tokenId": obj.get("tokenId"),
            "BAID": obj.get("BAID"),
            "email": obj.get("email"),
            "preapprovalKey": obj.get("preapprovalKey"),
            "applePaymentData": obj.get("applePaymentData"),
            "googlePaymentToken": obj.get("googlePaymentToken"),
            "type": obj.get("type")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
