# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from zuora_sdk.models.billing_adjustment_exclusion import BillingAdjustmentExclusion
from typing import Optional, Set
from typing_extensions import Self

class CreateBillingAdjustmentRequest(BaseModel):
    """
    CreateBillingAdjustmentRequest
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    account_number: Optional[StrictStr] = Field(default=None, description="The account number for which the adjustment is created.    **Note**: The account number should be of the subscription owner.  **Note**: Only one of accountNumber or subscriptionNumber should be provided. ", alias="accountNumber")
    charge_numbers: Optional[List[StrictStr]] = Field(default=None, description="An optional container to specify charge numbers in the subscription for which the adjustment needs to be created. ", alias="chargeNumbers")
    deferred_revenue_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for the deferred revenue, such as Monthly Recurring Liability. ", alias="deferredRevenueAccountingCode")
    end_date: date = Field(description="The end date of the adjustment, in `yyyy-mm-dd` format. This is inclusive. ", alias="endDate")
    exclusion: Optional[List[BillingAdjustmentExclusion]] = Field(default=None, description="The charge numbers and the corresponding dates for exclusion of adjustment. ")
    reason: Optional[StrictStr] = Field(default=None, description="The reason for the adjustment. ")
    recognized_revenue_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for the recognized revenue, such as Monthly Recurring Charges or Overage Charges. ", alias="recognizedRevenueAccountingCode")
    revenue_recognition_rule_name: Optional[StrictStr] = Field(default=None, description="The name of the revenue recognition rule governing the revenue schedule. ", alias="revenueRecognitionRuleName")
    start_date: date = Field(description="The start date of the adjustment, in `yyyy-mm-dd` format. This is inclusive. ", alias="startDate")
    subscription_number: Optional[StrictStr] = Field(default=None, description="The subscription number for which the adjustment is created.  **Note**: Only one of accountNumber or subscriptionNumber should be provided. ", alias="subscriptionNumber")
    type: Optional[StrictStr] = None
    credit_memo_custom_fields: Optional[Dict[str, Dict[str, Any]]] = Field(default=None, description="Container for custom fields of the Credit Memo. The custom fields of the Credit Memo can be defined during Create Adjustment ", alias="creditMemoCustomFields")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["accountNumber", "chargeNumbers", "deferredRevenueAccountingCode", "endDate", "exclusion", "reason", "recognizedRevenueAccountingCode", "revenueRecognitionRuleName", "startDate", "subscriptionNumber", "type", "creditMemoCustomFields"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateBillingAdjustmentRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in exclusion (list)
        _items = []
        if self.exclusion:
            for _item_exclusion in self.exclusion:
                if _item_exclusion:
                    _items.append(_item_exclusion.to_dict())
            _dict['exclusion'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateBillingAdjustmentRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "accountNumber": obj.get("accountNumber"),
            "chargeNumbers": obj.get("chargeNumbers"),
            "deferredRevenueAccountingCode": obj.get("deferredRevenueAccountingCode"),
            "endDate": obj.get("endDate"),
            "exclusion": [BillingAdjustmentExclusion.from_dict(_item) for _item in obj["exclusion"]] if obj.get("exclusion") is not None else None,
            "reason": obj.get("reason"),
            "recognizedRevenueAccountingCode": obj.get("recognizedRevenueAccountingCode"),
            "revenueRecognitionRuleName": obj.get("revenueRecognitionRuleName"),
            "startDate": obj.get("startDate"),
            "subscriptionNumber": obj.get("subscriptionNumber"),
            "type": obj.get("type"),
            "creditMemoCustomFields": obj.get("creditMemoCustomFields")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
