# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class GetBillingPreviewRunResponse(BaseModel):
    """
    get billingPreviewRun response
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    run_number: Optional[StrictStr] = Field(default=None, description="The run number of the billing preview run. ", alias="runNumber")
    assume_renewal: Optional[StrictStr] = Field(default=None, alias="assumeRenewal")
    batch: Optional[StrictStr] = Field(default=None, description="The customer batch included in the billing preview run.         **Note**: This field is not available if you set the `zuora-version` request header to `314.0` or later. ")
    batches: Optional[StrictStr] = Field(default=None, description="The customer batches included in the billing preview run.   **Note**: This field is only available if you set the `zuora-version` request header to `314.0` or later. ")
    charge_type_to_exclude: Optional[StrictStr] = Field(default=None, description="The charge types excluded from the forecast run. ", alias="chargeTypeToExclude")
    created_by_id: Optional[StrictStr] = Field(default=None, description="The ID of the user who created the billing preview run. ", alias="createdById")
    created_date: Optional[StrictStr] = Field(default=None, description="The date and time when the billing preview run was created. ", alias="createdDate")
    start_date: Optional[StrictStr] = Field(default=None, description="The date and time when the billing preview run starts. ", alias="startDate")
    end_date: Optional[StrictStr] = Field(default=None, description="The date and time when the billing preview run completes. ", alias="endDate")
    error_message: Optional[StrictStr] = Field(default=None, description="The error message generated by a failed billing preview run. ", alias="errorMessage")
    including_draft_items: Optional[StrictBool] = Field(default=None, description="Whether draft document items are included in the billing preview run. By default, draft document items are not included. ", alias="includingDraftItems")
    including_evergreen_subscription: Optional[StrictBool] = Field(default=None, description="Indicates if evergreen subscriptions are included in the billing preview run. ", alias="includingEvergreenSubscription")
    result_file_url: Optional[StrictStr] = Field(default=None, description="The URL of the zipped CSV result file generated by the billing preview run. This file contains the preview invoice item data and credit memo item data for the specified customers.  If the value of `storageOption` field is `Database`, the returned `resultFileUrl` field is null.  **Note:** The credit memo item data is only available if you have Invoice Settlement feature enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information. ", alias="resultFileUrl")
    status: Optional[StrictStr] = Field(default=None, description="The status of the >billing preview run.  **Possible values:**   * 0: Pending * 1: Processing * 2: Completed * 3: Error * 4: Canceled ")
    storage_option: Optional[StrictStr] = Field(default=None, alias="storageOption")
    succeeded_accounts: Optional[StrictInt] = Field(default=None, description="The number of accounts for which preview invoice item data and credit memo item data was successfully generated during the billing preview run.  **Note:** The credit memo item data is only available if you have Invoice Settlement feature enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information. ", alias="succeededAccounts")
    success: Optional[StrictBool] = Field(default=None, description="Returns `true` if the request was processed successfully.")
    target_date: Optional[date] = Field(default=None, description="The target date for the billing preview run.           ", alias="targetDate")
    total_accounts: Optional[StrictInt] = Field(default=None, description="The total number of accounts in the billing preview run. ", alias="totalAccounts")
    updated_by_id: Optional[StrictStr] = Field(default=None, description="The ID of the user who last updated the billing preview run. ", alias="updatedById")
    updated_date: Optional[StrictStr] = Field(default=None, description="The date and time when the billing preview run was last updated. ", alias="updatedDate")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["runNumber", "assumeRenewal", "batch", "batches", "chargeTypeToExclude", "createdById", "createdDate", "startDate", "endDate", "errorMessage", "includingDraftItems", "includingEvergreenSubscription", "resultFileUrl", "status", "storageOption", "succeededAccounts", "success", "targetDate", "totalAccounts", "updatedById", "updatedDate"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GetBillingPreviewRunResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetBillingPreviewRunResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "runNumber": obj.get("runNumber"),
            "assumeRenewal": obj.get("assumeRenewal"),
            "batch": obj.get("batch"),
            "batches": obj.get("batches"),
            "chargeTypeToExclude": obj.get("chargeTypeToExclude"),
            "createdById": obj.get("createdById"),
            "createdDate": obj.get("createdDate"),
            "startDate": obj.get("startDate"),
            "endDate": obj.get("endDate"),
            "errorMessage": obj.get("errorMessage"),
            "includingDraftItems": obj.get("includingDraftItems"),
            "includingEvergreenSubscription": obj.get("includingEvergreenSubscription"),
            "resultFileUrl": obj.get("resultFileUrl"),
            "status": obj.get("status"),
            "storageOption": obj.get("storageOption"),
            "succeededAccounts": obj.get("succeededAccounts"),
            "success": obj.get("success"),
            "targetDate": obj.get("targetDate"),
            "totalAccounts": obj.get("totalAccounts"),
            "updatedById": obj.get("updatedById"),
            "updatedDate": obj.get("updatedDate")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
