# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from zuora_sdk.models.get_workflow_response_tasks import GetWorkflowResponseTasks
from typing import Optional, Set
from typing_extensions import Self

class GetWorkflowResponse(BaseModel):
    """
    GetWorkflowResponse
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    cpu_time: Optional[StrictStr] = Field(default=None, description="The overall CPU time for the execution of the workflow. ", alias="cpuTime")
    created_at: Optional[StrictStr] = Field(default=None, description="The date and time when the workflow is created, in the `YYYY-MM-DD HH:MM:SS` format.. ", alias="createdAt")
    finished_at: Optional[StrictStr] = Field(default=None, description="The date and time when the execution of the workflow completes, in the `YYYY-MM-DD HH:MM:SS` format. ", alias="finishedAt")
    id: Optional[StrictInt] = Field(default=None, description="The unique ID of the workflow. ")
    messages: Optional[Dict[str, Any]] = Field(default=None, description="Messages from tasks.   **Note:** This field is only returned in Production. ")
    name: Optional[StrictStr] = Field(default=None, description="The unique run number of the workflow. ")
    original_workflow_id: Optional[StrictStr] = Field(default=None, description="The ID of the workflow setup. ", alias="originalWorkflowId")
    run_time: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The execution time of the workflow including the waiting time, in seconds. ", alias="runTime")
    status: Optional[StrictStr] = None
    tasks: Optional[GetWorkflowResponseTasks] = None
    type: Optional[StrictStr] = Field(default=None, description="The type of the current workflow. Possible values:   - `Workflow::Setup`: The workflow is a setup and is used for creating workflow instances.   - `Workflow::Instance`: The workflow is an execution that has data. ")
    updated_at: Optional[StrictStr] = Field(default=None, description="The date and time when the workflow is updated the last time, in the `YYYY-MM-DD HH:MM:SS` format. ", alias="updatedAt")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["cpuTime", "createdAt", "finishedAt", "id", "messages", "name", "originalWorkflowId", "runTime", "status", "tasks", "type", "updatedAt"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GetWorkflowResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of tasks
        if self.tasks:
            _dict['tasks'] = self.tasks.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetWorkflowResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "cpuTime": obj.get("cpuTime"),
            "createdAt": obj.get("createdAt"),
            "finishedAt": obj.get("finishedAt"),
            "id": obj.get("id"),
            "messages": obj.get("messages"),
            "name": obj.get("name"),
            "originalWorkflowId": obj.get("originalWorkflowId"),
            "runTime": obj.get("runTime"),
            "status": obj.get("status"),
            "tasks": GetWorkflowResponseTasks.from_dict(obj["tasks"]) if obj.get("tasks") is not None else None,
            "type": obj.get("type"),
            "updatedAt": obj.get("updatedAt")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
