# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from zuora_sdk.models.debit_memo_item_from_invoice_item_finance_information import DebitMemoItemFromInvoiceItemFinanceInformation
from zuora_sdk.models.update_debit_memo_tax_item_request import UpdateDebitMemoTaxItemRequest
from typing import Optional, Set
from typing_extensions import Self

class UpdateDebitMemoItem(BaseModel):
    """
    UpdateDebitMemoItem
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of the debit memo item. For tax-inclusive debit memo items, the amount indicates the debit memo item amount including tax. For tax-exclusive debit memo items, the amount indicates the debit memo item amount excluding tax.")
    comment: Optional[StrictStr] = Field(default=None, description="Comments about the debit memo item.")
    delete: Optional[StrictBool] = Field(default=None, description="Whether to delete the existing debit memo item. **Note**: This field is available only if id is not null.")
    exclude_item_billing_from_revenue_accounting: Optional[StrictBool] = Field(default=None, description="The flag to exclude the debit memo item from revenue accounting.  **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled.             ", alias="excludeItemBillingFromRevenueAccounting")
    finance_information: Optional[DebitMemoItemFromInvoiceItemFinanceInformation] = Field(default=None, alias="financeInformation")
    id: Annotated[str, Field(min_length=32, strict=True, max_length=32)] = Field(description="The ID of the debit memo item.")
    product_rate_plan_charge_id: Optional[StrictStr] = Field(default=None, description="The ID of the product rate plan charge that the debit memo is created from. **Note**: This field is available only if id is null.", alias="productRatePlanChargeId")
    quantity: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The number of units for the debit memo item.")
    service_end_date: Optional[date] = Field(default=None, description="The service end date of the debit memo item.", alias="serviceEndDate")
    service_start_date: Optional[date] = Field(default=None, description="The service start date of the debit memo item.  ", alias="serviceStartDate")
    sku_name: Optional[StrictStr] = Field(default=None, description="The name of the SKU.", alias="skuName")
    tax_items: Optional[List[UpdateDebitMemoTaxItemRequest]] = Field(default=None, description="Container for debit memo taxation items.", alias="taxItems")
    unit_of_measure: Optional[StrictStr] = Field(default=None, description="The definable unit that you measure when determining charges.", alias="unitOfMeasure")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["amount", "comment", "delete", "excludeItemBillingFromRevenueAccounting", "financeInformation", "id", "productRatePlanChargeId", "quantity", "serviceEndDate", "serviceStartDate", "skuName", "taxItems", "unitOfMeasure"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UpdateDebitMemoItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of finance_information
        if self.finance_information:
            _dict['financeInformation'] = self.finance_information.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in tax_items (list)
        _items = []
        if self.tax_items:
            for _item_tax_items in self.tax_items:
                if _item_tax_items:
                    _items.append(_item_tax_items.to_dict())
            _dict['taxItems'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UpdateDebitMemoItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "amount": obj.get("amount"),
            "comment": obj.get("comment"),
            "delete": obj.get("delete"),
            "excludeItemBillingFromRevenueAccounting": obj.get("excludeItemBillingFromRevenueAccounting"),
            "financeInformation": DebitMemoItemFromInvoiceItemFinanceInformation.from_dict(obj["financeInformation"]) if obj.get("financeInformation") is not None else None,
            "id": obj.get("id"),
            "productRatePlanChargeId": obj.get("productRatePlanChargeId"),
            "quantity": obj.get("quantity"),
            "serviceEndDate": obj.get("serviceEndDate"),
            "serviceStartDate": obj.get("serviceStartDate"),
            "skuName": obj.get("skuName"),
            "taxItems": [UpdateDebitMemoTaxItemRequest.from_dict(_item) for _item in obj["taxItems"]] if obj.get("taxItems") is not None else None,
            "unitOfMeasure": obj.get("unitOfMeasure")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
