# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from zuora_sdk.models.charge_metrics import ChargeMetrics
from zuora_sdk.models.preview_subscription_invoice import PreviewSubscriptionInvoice
from zuora_sdk.models.preview_subscription_invoice_item import PreviewSubscriptionInvoiceItem
from zuora_sdk.models.subscription_credit_memo import SubscriptionCreditMemo
from typing import Optional, Set
from typing_extensions import Self

class UpdateSubscriptionResponse(BaseModel):
    """
    UpdateSubscriptionResponse
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Invoice amount. Preview mode only. This field is in Zuora REST API version control. Supported minor versions are `206.0` and earlier. It is moved to this **invoice** container after 206.0 ")
    amount_without_tax: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Invoice amount minus tax. Preview mode only. This field is in Zuora REST API version control. Supported minor versions are `206.0` and earlier. It is moved to this **invoice** container after 206.0 ", alias="amountWithoutTax")
    charge_metrics: Optional[ChargeMetrics] = Field(default=None, alias="chargeMetrics")
    credit_memo: Optional[SubscriptionCreditMemo] = Field(default=None, alias="creditMemo")
    credit_memo_id: Optional[StrictStr] = Field(default=None, description="The credit memo ID, if a credit memo is generated during the subscription process.  **Note:** This container is only available if you set the Zuora REST API minor version to 207.0 or later in the request header, and you have  [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information. ", alias="creditMemoId")
    invoice: Optional[PreviewSubscriptionInvoice] = None
    invoice_id: Optional[StrictStr] = Field(default=None, description="Invoice ID, if an invoice is generated during the update. ", alias="invoiceId")
    invoice_items: Optional[List[PreviewSubscriptionInvoiceItem]] = Field(default=None, description="This field is only available REST API max version 206.0. Container for invoice items. ", alias="invoiceItems")
    invoice_target_date: Optional[date] = Field(default=None, description="Date through which charges are calculated on the invoice, as yyyy-mm-dd. Preview mode only.  **Note:** This field is only available if you do not specify the Zuora REST API minor version or specify the minor version to 186.0, 187.0, 188.0, 189.0, and 196.0. See [Zuora REST API Versions](https://www.zuora.com/developer/api-references/api/overview/#section/API-Versions) for more information. ", alias="invoiceTargetDate")
    paid_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Payment amount, if a payment is collected ", alias="paidAmount")
    payment_id: Optional[StrictStr] = Field(default=None, description="Payment ID, if a payment is collected. ", alias="paymentId")
    subscription_id: Optional[StrictStr] = Field(default=None, description="The ID of the resulting new subscription. ", alias="subscriptionId")
    success: Optional[StrictBool] = Field(default=None, description="Returns `true` if the request was processed successfully. ")
    target_date: Optional[date] = Field(default=None, description="Date through which to calculate charges if an invoice is generated, as yyyy-mm-dd. Default is current date.  **Note:** This field is only available if you set the Zuora REST API minor version to 207.0 or later in the request header. See [Zuora REST API Versions](https://www.zuora.com/developer/api-references/api/overview/#section/API-Versions) for more information. ", alias="targetDate")
    tax_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Tax amount on the invoice. This field is in Zuora REST API version control. Supported minor versions are `206.0` and earlier. It is moved to this **invoice** container after 206.0 ", alias="taxAmount")
    total_delta_mrr: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Change in the subscription monthly recurring revenue as a result of the update. ", alias="totalDeltaMrr")
    total_delta_tcv: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Change in the total contracted value of the subscription as a result of the update. ", alias="totalDeltaTcv")
    order_numbers: Optional[List[StrictStr]] = Field(default=None, description="The order numbers. It is available for Orders Tenants. ", alias="orderNumbers")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["amount", "amountWithoutTax", "chargeMetrics", "creditMemo", "creditMemoId", "invoice", "invoiceId", "invoiceItems", "invoiceTargetDate", "paidAmount", "paymentId", "subscriptionId", "success", "targetDate", "taxAmount", "totalDeltaMrr", "totalDeltaTcv", "orderNumbers"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UpdateSubscriptionResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of charge_metrics
        if self.charge_metrics:
            _dict['chargeMetrics'] = self.charge_metrics.to_dict()
        # override the default output from pydantic by calling `to_dict()` of credit_memo
        if self.credit_memo:
            _dict['creditMemo'] = self.credit_memo.to_dict()
        # override the default output from pydantic by calling `to_dict()` of invoice
        if self.invoice:
            _dict['invoice'] = self.invoice.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in invoice_items (list)
        _items = []
        if self.invoice_items:
            for _item_invoice_items in self.invoice_items:
                if _item_invoice_items:
                    _items.append(_item_invoice_items.to_dict())
            _dict['invoiceItems'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UpdateSubscriptionResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "amount": obj.get("amount"),
            "amountWithoutTax": obj.get("amountWithoutTax"),
            "chargeMetrics": ChargeMetrics.from_dict(obj["chargeMetrics"]) if obj.get("chargeMetrics") is not None else None,
            "creditMemo": SubscriptionCreditMemo.from_dict(obj["creditMemo"]) if obj.get("creditMemo") is not None else None,
            "creditMemoId": obj.get("creditMemoId"),
            "invoice": PreviewSubscriptionInvoice.from_dict(obj["invoice"]) if obj.get("invoice") is not None else None,
            "invoiceId": obj.get("invoiceId"),
            "invoiceItems": [PreviewSubscriptionInvoiceItem.from_dict(_item) for _item in obj["invoiceItems"]] if obj.get("invoiceItems") is not None else None,
            "invoiceTargetDate": obj.get("invoiceTargetDate"),
            "paidAmount": obj.get("paidAmount"),
            "paymentId": obj.get("paymentId"),
            "subscriptionId": obj.get("subscriptionId"),
            "success": obj.get("success"),
            "targetDate": obj.get("targetDate"),
            "taxAmount": obj.get("taxAmount"),
            "totalDeltaMrr": obj.get("totalDeltaMrr"),
            "totalDeltaTcv": obj.get("totalDeltaTcv"),
            "orderNumbers": obj.get("orderNumbers")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
